* file chap04.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all
capture log close

/*---Chapter 4.1 in POE5 Stata---*/    // This points to this manual’s sections

/*---POE5 Example 4.1---*/             // This points to POE5 sections

* open data
use stata/food, clear

* add observation 
edit
set obs 41
replace income=20 in 41

* estimate regression
quietly regress food_exp income
predict yhat
predict sef, stdf

* compute t-critical value
scalar df = e(df_r)
scalar tc = invttail(df,.025)
di "t critical value 97.5 percentile = " tc
gen lb = yhat - tc*sef
gen ub = yhat + tc*sef
list income lb yhat ub in 41
drop in 41

* using margins to create the prediction interval
margins, at(income = 20)
return list
matrix list r(b)
matrix marg = r(b)
scalar yhat = marg[1,1]

margins, at(income = 20) predict(stdf) nose force
matrix marg = r(b)
scalar sef = marg[1,1]
display "predicted mean for income = 20:"
display yhat
display "predictive interval for income = 20:"
display "(" yhat - tc*sef ", " yhat + tc*sef ")"

/*---Chapter 4.2 in POE5 Stata---*/
/*---POE5 Example 4.2---*/

* R2
pwcorr food_exp income yhat


/*---Chapter 4.3 in POE5 Stata---*/

/*---POE5 Section 4.3.1---*/

* effect of scaling

* create $ income and regress
gen inc_dollar = income*100
reg food_exp income
estimates store m1
reg food_exp inc_dollar
estimates store m2

/*---POE5 Example 4.3---*/

estimates table m1 m2, b se 

putdocx begin
putdocx table results = etable
putdocx save myresults.docx, replace

esttab m1 m2, b se
esttab m1 m2 using example4-3.rtf, replace  b se
esttab m1 m2, b se scalars(N r2 rss df_r) mtitles("m1" "m2")

/*---POE5 Example 4.4---*/
* Chapter 4.3.3 linear-log model

clear all

* open data
use stata/food

* log of income
gen lincome = ln(income)
label variable lincome "ln(income)"

* linear-log regression
reg food_exp lincome
predict lyhat
predict lehat, resid

* slope = beta2/x
summarize income
return list
scalar xbar = r(mean)
lincom lincome/xbar
lincom lincome/10
lincom lincome/20
lincom lincome/30

* fitted value plot
twoway (scatter food_exp income)                           ///
       (line lyhat income, sort lwidth(medthick)),         ///
       xtitle(Income) ytitle(Food Expenditure)             ///
       ylabel(0(100)600)                                   ///
       title(Figure 4.6 POE5) subtitle(Linear Log Model)   ///
       name(fig4_6, replace)

* linear relationship
quietly reg food_exp income
predict yhat
predict ehat, resid

* linear and linear-log fitted lines
twoway (scatter food_exp income)                                  ///
       (line lyhat income, sort lwidth(medthick))                 ///
       (line yhat income, sort lpattern(dash) lwidth(medthick)),  ///
       xtitle(Income) ytitle(Food Expenditure) ylabel(0(100)600)  ///
       title(Comparing models) name(fig4_6a, replace)             ///
       legend(order(2 "log-linear model" 3 "linear model"))
	   
/*---Chapter 4.4 in POE5 Stata---*/

/*---POE5 Example 4.5---*/

* plot linear-log model residuals	
twoway (scatter lehat income) ,                                   ///
       xtitle(Income) ytitle(Residuals)                           ///
       title(Figure 4.8 POE5) subtitle(Linear-Log Residuals)      ///
       name(fig4_8, replace)

quietly reg food_exp lincome
rvfplot, title(rvfplot) subtitle(resid vs. fitted)                ///
       yline(0) name(fig4_8a, replace)
rvpplot lincome, title(rvpplot) subtitle(resid vs. predictor)     ///
       yline(0) name(fig4_8b, replace)

/*---POE5 Example 4.5---*/

* analyze residuals from original equation

histogram ehat, percent title(histogram ehat) name(fig4_9a)
   
histogram ehat, percent width(20) title(Figure 4.9 POE5)          ///
       subtitle(Linear Model Residuals) name(fig4_9, replace)
	   
histogram lehat, percent bin(10) title(Log-linear Model Residuals) ///
       name(fig4_9b, replace)

/*---POE5 Example 4.6---*/

* Jarque-Bera test of error normality
summarize ehat, detail
return list

scalar jb = (r(N)/6)*( (r(skewness)^2) + ((r(kurtosis)-3)^2)/4 )
di "Jarque-Bera Statistic = " jb

scalar chic = invchi2tail(2,.05)
di "Chi-square(2) 95th percentile = " chic

scalar pvalue = chi2tail(2,jb)
di "Jarque-Bera p-value = " pvalue

program jbprog
    scalar jb = (r(N)/6)*( (r(skewness)^2) + ((r(kurtosis)-3)^2)/4 )
    di "Jarque-Bera Statistic = " jb
    scalar chic = invchi2tail(2,.05)
    di "Chi-square(2) 95th percentile = " chic
    scalar pvalue = chi2tail(2,jb)
    di "Jarque-Bera p-value = " pvalue
end

jbprog

sktest ehat

/*---POE5 Example 4.7---*/
* influential observations

quietly reg food income
scalar k = e(df_m)+1
scalar nobs = e(N)

predict h, leverage
di "leverage average   " k/nobs
list h income if h > 2*k/nobs

predict ehatstu, rstudent
list ehatstu ehat food_exp income if abs(ehatstu) > 2

predict ehatstd, rstandard
list ehatstd ehat food_exp income if abs(ehatstd) > 2

lvr2plot, title(leverage vs resid^2) name(lvr2, replace)

gen hhid = _n
label var hhid "Observation id for each household"
lvr2plot, title(leverage vs resid^2) name(lvr2a, replace) mlabel(hhid) 

predict dfbeta2, dfbeta(income)
di "dfbetas threshold " 2/sqrt(nobs)
list dfbeta2 food_exp income if abs(dfbeta2) > 2/sqrt(nobs)

dfbeta

predict dfits, dfits
di "dfits threshold " 2*sqrt(k/nobs)
list dfits yhat food_exp income if abs(dfits) > 2*sqrt(k/nobs)

/*---Chapter 4.4 in POE5 Stata---*/

/*---POE5 Example 4.8---*/ 

* Polynomial model Chapter 4.4

clear all

* open data and examine
use stata/wa_wheat
describe
summarize

gen yield = greenough
label variable yield "wheat yield greenough shire"

generate year = time + 1949
tsset year

* plot data
twoway (scatter yield year, msymbol(circle)) ,             ///
       xtitle(Year) ylabel(0(.5)2.5) ytitle(Yield)         ///
       title(Figure 4.10 POE5) subtitle(Wheat Yield)       ///
       name(fig4_10, replace)

* regression     
reg yield time
predict yhat
predict ehat, residuals

* plot fitted lines and data
twoway (scatter yield year)                                ///
       (line yhat year, lwidth(medthick)) ,                ///
       xtitle(Year) ytitle(Yield) ylabel(0(.5)2.5)         ///
       title(Wheat Yield Fitted Linear Model)              ///
       name(cubic, replace)
	   
twoway (scatter yield year)                                ///
       (line yhat year, lwidth(medthick)) ,                ///
       xtitle(Year) ytitle(Yield) ylabel(0(.5)2.5)         ///
       title(Wheat Yield Fitted Linear Model)              ///
       name(cubic, replace)

* plot residuals
twoway (scatter ehat year),                                ///
       xtitle(Year) ytitle(Residuals) yline(0)             ///
       title(Figure 4.11 POE5)                             ///
       subtitle(Linear Model Residuals) name(fig4_11, replace)

rvpplot time, recast(bar) yline(0) name(fig4_11b, replace)

* Chapter 4.4.2 Cubic equation for yield

* create scaled cubic variable
generate time0=time/100
list yield time0 in 1/5
summarize time0

* cubic regression
reg yield c.time0#c.time0#c.time0
predict yhat3
predict ehat3, residuals

* slopes
margins, dydx(*) at(time=(0.15 0.30 0.45))

* plot fitted lines and data
twoway (scatter yield year)                                ///
       (line yhat3 year, lwidth(medthick)) ,               ///
       xtitle(Year) ytitle(Yield) ylabel(0(.5)2.5)         ///
       title(Wheat Yield Fitted Cubic Model)               ///
       name(cubic2, replace)

* plot residuals
twoway (scatter ehat3 year) ,                               ///
       xtitle(Year) ytitle(Residuals) yline(0)              ///
       title(Figure 4.12 POE5)                              ///
       subtitle(Cubic model residuals) name(fig4_12, replace)

* Chapter 4.5 Log-linear Models

/*---POE5 Example 4.9---*/

* Wheat growth model
gen lyield = ln(yield)
label var lyield "ln(yield)"
reg lyield time
lincom 100*time

reg lyield year
lincom 100*year

/*---Chapter 4.5 in POE5 Stata---*/    //
/*---POE5 Example 4.10---*/
* Wage Equation

clear all

* open cps4_small data
use stata/cps5_small

* summarize and plot
describe wage educ
summarize wage educ
tabulate educ

twoway (scatter wage educ, msize(small) color(%20)),    ///
       xtitle(Education) ytitle(Wage)                   ///
       title(Wage-Education Scatter)                    ///
       name(wage_educ, replace)

* create log(wage) and plot
gen lwage = ln(wage)
label variable  lwage "ln(wage)"
twoway (scatter lwage educ, msize(small) color(%20)),  ///
       xtitle(Education) ytitle(ln(Wage))              ///
       title(ln(Wage)-Education Scatter)               ///
       name(lwage_educ, replace)

* log-linear regression
* add one observation

edit 
set obs 1201
replace educ=12 in 1201


reg lwage educ
predict lwagehat
predict ehat, residuals
predict sef, stdf

* calculate sigma-hat^2
ereturn list
scalar sig2 = e(rmse)^2
di "sigma-hat squared = " sig2
scalar df = e(df_r)

* Analyze resdiduals
histogram ehat, percent title(ln(Wage) Model Residuals) ///
          name(ehat_hist, replace)

summarize ehat, detail

program jbprog
    scalar jb = (r(N)/6)*( (r(skewness)^2) + ((r(kurtosis)-3)^2)/4 )
    di "Jarque-Bera Statistic = " jb
    scalar chic = invchi2tail(2,.05)
    di "Chi-square(2) 95th percentile = " chic
    scalar pvalue = chi2tail(2,jb)
    di "Jarque-Bera p-value = " pvalue
end

jbprog

rvpplot educ, yline(0)

/*---POE5 Example 4.11---*/

* compute natural and corrected predictor and plot
gen yhatn = exp(lwagehat)
label var yhatn "fitted wage, natural"

di "correction factor = " exp(sig2/2)
gen yhatc = yhatn*exp(sig2/2)
label var yhatc "fitted wage, corrected"

twoway (scatter wage educ, msize(small) color(%20)) 	         ///
       (line yhatn educ, sort lwidth(medthick) lpattern(dash))   ///
       (line yhatc educ, sort lwidth(medthick) lpattern(solid)), ///
       title(Figure 4.13 POE5) subtitle(Wage Predictors)         ///
       legend(order(2 "natural" 3 "corrected"))                  ///
       name(lwage_predict, replace)                                                 
	  
* list predicted values

list educ yhatn yhatc in 1201
summarize wage if educ==12 in 1/1200

* using margins for predictions 
margins, at(educ=12) expression(exp(xb()))
margins, at(educ=12) expression(exp(xb())*exp(sig2/2))

* R^2
pwcorr wage yhatn yhatc
di "r2g = " r(rho)^2

* prediction interval
scalar tc = invttail(df,.025)
gen lb_lwage = lwagehat - tc*sef
gen ub_lwage = lwagehat + tc*sef
gen lb_wage = exp(lb_lwage)
gen ub_wage = exp(ub_lwage)

* list and plot
list lb_wage ub_wage in 1201
twoway (scatter wage educ, msize(small) color(%20))                     ///
(line yhatn educ, sort lcolor(black) lwidth(medthick) lpattern(solid))  ///
(line ub_wage educ, sort lcolor(blue) lwidth(medthick) lpattern(dash))  ///
(line lb_wage educ, sort lcolor(blue) lwidth(medthick) lpattern(dash)), ///
title(Figure 4.14 POE5) subtitle(95% prediction interval for wage)      ///
name(lwage_interval, replace)

* using margins for prediction interval
margins, at(educ=12)                             // fitted lwage
matrix marg = r(b)                               // stored value
scalar hat = marg[1,1]                           // name it

margins, at(educ=12) predict(stdf) nose force    // se of forecast
matrix marg = r(b)
scalar stdf = marg[1,1]

display "Prediction interval for educ = 12:"
display "(" exp(hat - tc*stdf) ", " exp(hat + tc*stdf) ")"

/*---POE5 Example 4.11---*/

clear all

* A log-log model example
use stata/newbroiler
describe p q
summarize p q

gen lq = ln(q)
label variable lq "ln(q)"
gen lp = ln(p)
label variable lp "ln(p)"
reg lq lp
predict lqhat
scalar sig2 = e(rmse)^2
gen qhatc = exp(lqhat)*exp(sig2/2)
label variable qhatc "predicted Q, corrected)"

twoway (scatter q p, msize(small) lwidth(medthick) lpattern(solid)) ///
       (line qhatc p, sort lwidth(medthick)),                       ///
       xtitle(Price of Chicken) ytitle(Quantity of Chicken)         ///
       title(Figure 4.15 POE5) subtitle(Poultry Demand)             ///
       legend(order(1 "Q" 2 "predicted Q"))                         ///   
       name(fig4_15, replace)

pwcorr q qhatc
di "r2g = " r(rho)^2

