* file chap06.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 5e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1                  
* cd ~/Documents/poe5         // use to change working directory
clear all                     // use this option if previous work can be erased
capture log close 

* Utility to add squares to data set
capture program drop makesquare
program define makesquare
   syntax varlist
   foreach var of local varlist {
      capture drop sq_`var'
      gen sq_`var' = `var'^2
      label variable sq_`var' "`var' squared"
      }
end

* Utility to add logs to dataset
capture program drop makelogs
program define makelogs
   syntax varlist
   foreach var of local varlist {
      capture drop l`var'
      gen l`var' = ln(`var')
      label variable l`var' "ln(`var')"
      }
end

* Computing RMSE
capture program drop rmse
program define rmse
  syntax varlist(numeric fv ts) [if] [in] [,t1(integer 1) t2(integer 1)]
  quietly {
    local depvar:word 1 of `varlist'
	local indepvar:list varlist-depvar
    regress `varlist' `in'
    tempvar yhat sq_diff 
    predict `yhat', xb
    gen `sq_diff' = (`depvar'-`yhat')^2
    summarize `sq_diff' in `t1'/`t2'
    local rmse = sqrt(r(mean))
    }
  display " Dependent Variable: `depvar' "
  display " Independent Variables: `indepvar' "
  display " RMSE: `rmse'"
 end
  
* Model selection program
capture program drop modelsel
program define modelsel
  syntax varlist(numeric fv ts) [if] [in]
  tempname aic sc
  quietly {
    local depvar:word 1 of `varlist'
    local indepvar:list varlist-depvar
    regress `varlist' `in'
    scalar `aic' = ln(e(rss)/e(N))+2*e(rank)/e(N) 
    scalar `sc' = ln(e(rss)/e(N))+e(rank)*ln(e(N))/e(N)
    }
  display " Dependent Variable: `depvar' "
  display " Independent Variables: `indepvar' "
  display " R-square = "e(r2)
  display " Adjusted r_square " e(r2_a)
  display " AIC = " `aic'
  display " SC = " `sc'
end

* Regression specification test (RESET)
capture program drop reset 
program define reset
   syntax varlist
   quietly {
      regress `varlist'
      tempvar yhat yhat2 yhat3
      predict `yhat'
      gen `yhat2' = `yhat'*`yhat'
      gen `yhat3' = `yhat'^3
      regress `varlist' `yhat2'
      test `yhat2'
      local p_reset1 = r(p) 
      regress `varlist' `yhat2' `yhat3'  
      test `yhat2' `yhat3'
      local p_reset2 = r(p)
      }
   display "p-value for RESET(1): `p_reset1'"
   display "p-value for RESET(2): `p_reset2'"
end
   
* Confidence Interval
capture program drop ci_t
program define ci_t
 version 15.1
 args b se df a  //b=estimate, se=std error, df=d of f, a=alpha/2
 tempname fL fU
 scalar `fL' = `b' - invttail(`df',`a')*`se'
 scalar `fU' = `b' + invttail(`df',`a')*`se'
 display "The " 1-2*`a' " Confidence Interval is (" `fL' ", " `fU' ")"  
end

* -------------------------------------------
* Thedu following block estimates Andy's sales
* and uses thedu difference in SSE to test 
* a hypothedusis using an F-statistic
* -------------------------------------------

use stata/andy, clear

/*---POE5 Example 6.1---*/
* Testing the Effect of Advertising
 
* Unrestricted Model
regress sales price advert c.advert#c.advert
scalar sseu = e(rss)
scalar df_unrest = e(df_r)

* Restricted Model
regress sales price
scalar sser = e(rss)
scalar df_rest = e(df_r)
scalar J = df_rest - df_unrest

* F-statistic, critical value, pvalue
scalar fstat = ((sser - sseu)/J)/(sseu/(df_unrest))
scalar crit1 = invFtail(J,df_unrest,.05)
scalar pvalue = Ftail(J,df_unrest,fstat)

scalar list sseu sser J df_unrest fstat pvalue crit1

/*---POE5 Example 6.2---*/
* The F-Test Procedure

* -------------------------------------------
* Here, we use Stata's test statement 
* to test hypothedusis using an F-statistic
* Note: Four versions of the syntax
* -------------------------------------------

regress sales price advert c.advert#c.advert
testparm advert c.advert#c.advert
testparm c.advert##c.advert
test (advert)(c.advert#c.advert)
test (_b[advert]=0)(_b[c.advert#c.advert]=0)

/*---POE5 Example 6.3---*/ 
* Overall Significance of Burger Barns Equation
* Uses same Unrestricted Model as above

* Unrestricted Model (all variables)
regress sales price advert c.advert#c.advert
scalar sseu = e(rss)
scalar df_unrest = e(df_r)

* Restricted Model (no explanatory variables)
regress sales 
scalar sser = e(rss)
scalar df_rest = e(df_r)
scalar J = df_rest - df_unrest

* F-statistic, critical value, pvalue
scalar fstat = ((sser-sseu)/J)/(sseu/(df_unrest))
scalar crit2 = invFtail(J,df_unrest,.05)
scalar pvalue = Ftail(J,df_unrest,fstat)

scalar list sseu sser J df_unrest fstat pvalue crit2

/*---POE5 Example 6.4---*/ 
* When are t- and F-tests equivalent?
* Unrestricted Regression
regress sales price c.advert##c.advert
scalar sseu = e(rss)
scalar df_unrest = e(df_r)

scalar tratio = _b[price]/_se[price]
scalar t_sq = tratio^2

* Restricted Regression
regress sales c.advert##c.advert
scalar sser = e(rss)
scalar df_rest = e(df_r)
scalar J = df_rest - df_unrest

* F-statistic, critical value, pvalue
scalar fstat = ((sser-sseu)/J)/(sseu/(df_unrest))
scalar crit = invFtail(J,df_unrest,.05)
scalar pvalue = Ftail(J,df_unrest,fstat)

scalar list sseu sser J df_unrest fstat pvalue crit tratio t_sq

/*---POE5 Example 6.5---*/ 
* Testing Optimal Advertising
* Uses both sets of syntax for test

regress sales price c.advert##c.advert
test _b[advert]+3.8*_b[c.advert#c.advert]=1
test advert+3.8*c.advert#c.advert=1

* t stat for Optimal Advertising
lincom _b[advert]+3.8*_b[c.advert#c.advert]-1
lincom advert+3.8*c.advert#c.advert-1
scalar t = r(estimate)/r(se)
scalar pvalue = 2*ttail(e(df_r),abs(t))
scalar list t pvalue

/*---POE5 Example 6.6---*/  
* A One-Tail Test
scalar pval = ttail(e(df_r),t)
scalar crit = invttail(e(df_r),.05)
scalar list t pval crit

/*---POE5 Example 6.7---*/ 
* Two (J = 2) Complex Hypotheses
regress sales price advert c.advert#c.advert
test (_b[advert]+3.8*_b[c.advert#c.advert]=1) (_b[_cons]+ ///
	 6*_b[price]+1.9*_b[advert]+3.61*_b[c.advert#c.advert]= 80)
	 
test (advert+3.8*c.advert#c.advert=1) (_cons + 6*price + /// 
     1.9*advert+3.61*c.advert#c.advert= 80)

/*---POE5 Examples 6.2 and 6.5---*/ 
* Revisited
regress sales price c.advert##c.advert
scalar sseu = e(rss)
scalar df_unrest = e(df_r)

* Restricted Model
regress sales price
scalar sser = e(rss)
scalar df_rest = e(df_r)
scalar J = df_rest - df_unrest

scalar fstat = ((sser - sseu)/J)/(sseu/(df_unrest))
scalar Fcrit = invFtail(J,df_unrest,.05)
scalar Fpvalue = Ftail(J,df_unrest,fstat)

scalar chi2stat = (sser - sseu)/(sseu/(df_unrest))
scalar chi2crit = invchi2tail(J,.05)
scalar chi2pvalue = chi2tail(J,chi2stat)

scalar list fstat Fpvalue Fcrit
scalar list chi2stat chi2crit chi2pvalue

regress sales price c.advert##c.advert
test advert+3.8*c.advert#c.advert=1

scalar stat=r(F)
scalar J=r(df)
scalar df = r(df_r)
scalar Fpvalue = Ftail(J,df,stat)	 
scalar chi2pvalue = chi2tail(J,J*stat)
	 
scalar list stat J df Fpvalue chi2pvalue

/*---POE5 Example 6.8---*/ 
* A Nonlinear Hypothesis
regress sales price c.advert##c.advert
test advert+3.8*c.advert#c.advert=1
testnl (1-_b[advert])/(2*_b[c.advert#c.advert])=1.9
nlcom (1-_b[advert])/(2*_b[c.advert#c.advert])-1.9

/*---POE5 Example 6.9---*/ 
* Restricted Least Squares
use stata/beer, clear
makelogs q pb pl pr i 

constraint 1 lpb+lpl+lpr+li=0
cnsreg lq lpb lpl lpr li, c(1)

/*---POE5 Example 6.10---*/ 
* Family Income Equation
use stata/edu_inc, clear
makelogs faminc

regress lfaminc hedu wedu
regress lfaminc hedu

* correlations among regressors
pwcorr

/*---POE5 Example 6.11---*/ 
* Adding Children Aged Less Than 6 Years
regress lfaminc hedu wedu kl6

/*---POE5 Example 6.12---*/ 
* Adding Irrelevant Variables
regress lfaminc hedu wedu kl6 xtra_x5 xtra_x6

regress lfaminc hedu wedu
modelsel lfaminc hedu wedu

di "Model 1 (hedu) "
modelsel lfaminc hedu
estimates store Model1

di "Model 2 (hedu, wedu) "
modelsel lfaminc hedu wedu
estimates store Model2

di "Model 3 (hedu, wedu, kl6) "
modelsel lfaminc hedu c.wedu#c.hedu kl6
estimates store Model3

di "Model 4 (hedu, wedu, kl6, xtra_x5, xtra_x6) "
modelsel lfaminc hedu wedu kl6 xtra_x5 xtra_x6
estimates store Model4

di "Model 5 (hedu, kl6, xtra_x5, xtra_x6) "
modelsel lfaminc hedu kl6 xtra_x5 xtra_x6
estimates store Model5

estimates table Model1 Model2 Model3 Model4 Model5, b(%9.3f) ///
          stfmt(%9.3f) se stats(N r2 r2_a aic bic)
esttab Model1 Model2 Model3 Model4 Model5, compress b(%9.3f) ///
         se(%9.3f) r2 ar2 aic bic

estimates table M*, b(%9.3f) stfmt(%9.3f) se stats(N r2 r2_a aic bic)

esttab M*, compress b(%9.3f) se(%9.3f) r2 ar2 aic bic
		 
/*---POE5 Example 6.14---*/ 		 
* Applying RESET to Family Income Equation
regress lfaminc hedu wedu kl6
predict yhat, xb
gen yhat2 = yhat^2
gen yhat3 = yhat^3
regress lfaminc hedu wedu kl6 yhat2
test yhat2
regress lfaminc hedu wedu kl6 yhat2 yhat3
testparm yhat2 yhat3 

* -------------------------------------------
* Stata uses the estat ovtest following
* a regression to do a RESET(3) test.  
* -------------------------------------------

regress faminc hedu wedu kl6
estat ovtest 

* Reset using the RESET Program  
reset lfaminc hedu wedu
reset lfaminc hedu
reset lfaminc hedu wedu kl6
reset lfaminc hedu wedu kl6 xtra_x5 xtra_x6
reset lfaminc hedu kl6 xtra_x5 xtra_x6

* Table 6.2
pwcorr lfaminc hedu wedu kl6 xtra_x5 xtra_x6

/*---POE5 Example 6.13---*/ 
* A Control Variable for Ability
  
use stata/koop_tobias_87, clear
makelogs wage
reg lwage educ c.exper##c.exper 
reg lwage educ c.exper##c.exper score

/*---POE5 Example 6.15---*/ 
* Forecasting SALES for the Burger Barn
use stata/andy, clear
reg sales price c.advert##c.advert

clear
set obs 1
gen price = 6
gen advert = 1.9
predict yhat, xb
predict se_predict, stdp
predict se_forecast, stdf
list se_predict se_forecast

scalar lb = yhat - invttail(e(df_r),.025)*se_forecast
scalar ub = yhat + invttail(e(df_r),.025)*se_forecast
di "The 95% confidence interval is (" lb ", " ub ")"  

ci_t yhat se_forecast e(df_r) .025          // Explained in chapter 9

use stata/andy, clear
reg sales price c.advert##c.advert
matrix cov=e(V)
matlist cov

scalar sig2hat = e(rss)/e(df_r)
scalar list sig2hat
scalar forecast_var = sig2hat + cov[4,4] + 6^2*cov[1,1]     ///
   + 1.9^2*cov[2,2] + 3.61^2*cov[3,3] + 2*6*cov[4,1]        ///
   + 2*1.9*cov[4,2] + 2*3.61*cov[4,3] + 2*1.9*3.61*cov[2,3] ///
   + 2*6*1.9*cov[1,2] + 2*6*3.61*cov[1,3]
scalar list forecast_var

lincom _cons + 6*price + 1.9*advert + 3.61*c.advert#c.advert

/*---POE5 Example 6.16---*/ 
* Predicting House Prices
use stata/br5, clear
makelogs price
makesquare age sqft

modelsel lprice sqft age 
rmse lprice sqft age in 1/800,                           ///
            t1(801) t2(900) 

modelsel lprice sqft age sq_age
rmse lprice sqft age c.age#c.sqft in 1/800,              ///
            t1(801) t2(900)

modelsel lprice c.sqft##c.sqft age
rmse lprice c.sqft##c.sqft age in 1/800,                 ///
            t1(801) t2(900)

modelsel lprice sqft age c.age#c.sqft
rmse lprice sqft age c.age#c.sqft in 1/800,              ///
             t1(801) t2(900)
 
modelsel lprice sqft age c.age#c.age c.sqft#c.sqft
rmse lprice c.age##c.age c.sqft##c.sqft in 1/800,        /// 
             t1(801) t2(900)

modelsel lprice sqft c.age##c.age c.age#c.sqft
rmse lprice sqft c.age##c.age c.age#c.sqft in 1/800,     /// 
            t1(801) t2(900)

modelsel lprice c.sqft##c.sqft age c.age#c.sqft 
rmse lprice c.sqft##c.sqft age c.age#c.sqft in 1/800,    ///
            t1(801) t2(900) 

modelsel lprice c.age##c.age c.sqft##c.sqft c.age#c.sqft
rmse lprice c.age##c.age c.sqft##c.sqft c.age#c.sqft     ///
            in 1/800, t1(801) t2(900)

/*---POE5 Example 6.17---*/ 
* Collinearity in a Rice Production Function
use stata/rice5, clear
summarize, separator(0)
pwcorr

makelogs area fert labor prod
pwcorr l*

reg lprod larea llabor lfert if year==1994
estat vif

constraint 1 larea + llabor + lfert =1
cnsreg lprod larea llabor lfert if year==1994, c(1)

reg lprod larea llabor lfert
estat vif

/*---POE5 Example 6.18---*/ 
* Influential Observations in the House Price Equation
use stata/br5, clear
gen obs_number = _n
makelogs price
makesquare age

reg lprice sqft c.age##c.age 

predict dfb_sqft, dfbeta(sqft)
predict dfb_age, dfbeta(age)
predict dfb_age2, dfbeta(c.age#c.age)

list obs_number sqft age dfb_sqft if abs(dfb_sqft) > 2/sqrt(e(N))
list obs_number sqft age dfb_age if abs(dfb_age) > 2/sqrt(e(N))
list obs_number sqft age dfb_age2 if abs(dfb_age2) > 2/sqrt(e(N))

gen dfb_sqft_high = abs(dfb_sqft)
gen dfb_age_high = abs(dfb_age) 
gen dfb_age2_high = abs(dfb_age2)
 
sort dfb_sqft_high
list obs_number sqft age dfb_sqft in -5/-1
 
sort dfb_age_high
list obs_number sqft age dfb_age in -5/-1

sort dfb_age2_high
list obs_number sqft age dfb_age2 in -5/-1
 
predict student_res, rstudent
gen student_res_high = abs(student_res)
sort student_res_high
list obs_number sqft age student_res in -5/-1

predict lever, leverage
gen lever_high = abs(lever)
sort lever_high
list obs_number sqft age lever in -5/-1

predict dffits, dfits
gen dffits_high = abs(dffits)
sort dffits_high
list obs_number sqft age dffits in -5/-1

/*---POE5 Example 6.19---*/ 
* Nonlinear Least Squares Estimates for Simple Model
use stata/nlls, clear
nl (y={b}*x1+{b}^2*x2), variables(x1 x2)

/*---POE5 Example 6.20---*/
* A Logistic Growth Curve
use stata/steel, clear
nl (eaf = {alpha}/(1 + exp(-{beta}-{delta}*t))), variables(eaf t)
nl, coeflegend
display "Inflection Point " -_b[beta:_cons]/_b[delta:_cons]
nlcom -_b[beta:_cons]/_b[delta:_cons]
