* file chap08.do for Using Stata for Principles of Econometrics, 5e


* Stata Do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 4e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all
capture log close

/*---POE5 Example 8.1---*/
* Heteroskedasticity in the Food Expenditure Model
* OLS, OLS with White's std errors, GLS

use stata/food

regress food_exp income
predict ehat, res
label variable ehat "OLS resid"

graph twoway (scatter food_exp income)               ///
     (lfit food_exp income, lw(medthick)) ,          ///
      xtitle(Income) ytitle(Food expenditures)       ///
      ylabel(0(100)300)                              ///
      xlabel(0(5)40)                                 ///
      title(Figure 2.8 POE5) subtitle(Food expenditure regression) ///
      name(fig2_8, replace)
  
* Graph relationship between size of errors and income
generate abs_e = abs(ehat)
twoway (scatter abs_e income),                       ///
      xtitle(Income) ytitle(abs(ehat))               ///
      ylabel(0(100)300)                              ///
      xlabel(0(5)35)                                 ///
      title(Figure 8.2 POE5) subtitle(Absolute residuals) ///
      name(fig8_2, replace)

* Graph relationship between errors and income
graph twoway scatter ehat income, yline(0)           ///
      xtitle(Income) ytitle(ehat)                    ///
      ylabel(-300(100)300)                           ///
      xlabel(0(5)35)                                 ///
      title(Figure 8.3 POE5) subtitle(Food expenditure residuals) ///
      name(fig8_3, replace)

/*---POE5 Example 8.2---*/
* Robust Standard Errors in the Food Expenditure Model

quietly reg food_exp income
estimates store m1

quietly reg food_exp income, vce(robust)
estimates store m2

esttab m1 m2, b(%7.4f) se(%7.3f) scalars(F)        ///
       mtitles("Incorrect" "Robust") 
   
esttab m1 m2, b(%7.4f) ci(%7.3f) nostar scalars(F) ///
       mtitles("Incorrect" "Robust") 

estimates table m1 m1,  b(%7.4f) se(%7.3f) stats(F)

/*---POE5 Example 8.3---*/
* Applying GLS/WLS to the Food Expenditure Data
* the hard way
gen ystar = food_exp/sqrt(income)
gen x1star = 1/sqrt(income)
gen x2star = income/sqrt(income)
reg ystar x1star x2star, noconstant
predict estar, resid
label variable estar "Transformed model resid"

* using WLS
reg food_exp income [aweight=1/income]
estimates store m3
predict ehat_gls, resid
label variable ehat_gls "WLS resid"
gen ehat2 = ehat_gls/sqrt(income)
label variable ehat2 "Transformed WLS resid"

* residuals
list estar ehat_gls ehat2 in 1/5

* robust GLS
reg food_exp income [aweight=1/income], vce(robust)
estimates store m4

esttab m2 m3 m4, b(%7.4f) se(%7.3f) nostar scalars(F) ///
       mtitles("OLS Robust" "GLS" "GLS Robust") 

* check transformed model residuals 
twoway (scatter estar x2star),              ///
       xtitle(transformed Income)           ///
       ytitle(transformed model residuals)  ///
       name(fig8_4a, replace)

twoway (scatter estar income),              ///
       xtitle(Income)                       ///
       ytitle(transformed model residuals)  ///
       name(fig8_4b, replace)

/*---POE5 Example 8.4---*/
* Multiplicative Heteroskedasticity in the Food Expenditure Model
* the hard way
gen z2 = ln(income)
gen lehat_sq = ln(ehat^2)
reg lehat_sq z2
predict sighat, xb
gen wt = exp(sighat)
regress food_exp income [aweight=(1/wt)]

* using automatic command
hetregress food_exp income, twostep het(z2)
hetregress food_exp income, het(z2)

/*---POE5 Example 8.5---*/
* A heteroskedastic partition

clear all
use stata/cps5_small
keep if midwest==1

* summary stats
label def metro 0 "rural" 1 "metropolitan"
label val metro metro
bysort metro: summarize wage educ exper

* OLS robust
regress wage educ exper metro, vce(robust)
predict ehat, resid
gen lehat2 = ln(ehat^2)
reg lehat2 metro
gen h = exp(_b[metro]*metro)
regress wage educ exper metro [aweight=1/h]
regress wage educ exper metro [aweight=1/h], vce(robust)

* using hetregress
hetregress wage educ exper metro, twostep het(metro)
hetregress wage educ exper metro, het(metro) 

/*---POE5 Example 8.6---*/
* The Goldfeld–Quandt Test with Partitioned Data

clear all
use stata/cps5_small
keep if midwest==1

* Rural subsample regression
regress wage educ exper if metro == 0 
scalar rmse_r = e(rmse)
scalar df_r = e(df_r)
scalar sig2_r = rmse_r^2

* Urban subsample regression
regress wage educ exper if metro == 1 
scalar rmse_m = e(rmse)
scalar df_m = e(df_r)
scalar sig2_m = rmse_m^2

scalar GQ = rmse_m^2/rmse_r^2
scalar F975 = invFtail(df_m,df_r,.025)
scalar F025 = invFtail(df_m,df_r,.975)
scalar list sig2_m sig2_r GQ F025 F975

/*---POE5 Example 8.7---*/
* The Goldfeld–Quandt Test in the Food Expenditure Model

clear all
use stata/food
sort income, stable
reg food income in 1/20
scalar sig2_1 = e(rmse)^2
scalar df1 = e(df_r)
reg food income in 21/40
scalar sig2_2 = e(rmse)^2
scalar df2 = e(df_r)
scalar GQ = sig2_2/sig2_1
scalar F95 = invFtail(df2,df1,.05)

scalar list sig2_1 sig2_2 df1 df2 F95 GQ

/*---POE5 Example 8.8---*/
* Variance Stabilizing Log-transformation

clear all
use stata/cex5_small

keep if entert>0

hist entert, percent name(hist_ent, replace)                  ///
     xtitle("ENTERT") title("POE5 Figure 8.7(a)") 
 
gen lentert = ln(entert)
label variable lentert "ln(entert)"
hist lentert, percent name(hist_lent, replace)                 ///
      xtitle("ln(ENTERT)") title("POE5 Figure 8.7(b)") 

twoway (scatter entert income,  msymbol(smcircle) color(%20))  ///
       (lfit entert income, lwidth(medthick)),                 ///
        ytitle("ENTERT") xtitle("INCOME")                      ///
        title("POE5 Figure 8.8(a)" ) subtitle( "Linear model") ///
        legend(off) name(fig8_8a, replace)

* linear model
reg entert income college advanced

estat hettest income, iid

predict ehat, resid
gen ehat2 = ehat^2
reg ehat2 income
di "BP test " e(N)*e(r2)
di "0.05 critical value " invchi2tail(e(df_m),.05)

reg ehat2 (c.income college advanced)##(c.income college advanced)
di "White test " e(N)*e(r2)
di "0.05 critical value " invchi2tail(e(df_m),.05)

qui reg entert income college advanced
estat imtest, white

* log-linear model
twoway (scatter lentert income,  msymbol(smcircle) color(%20))  ///
       (lfit lentert income, lwidth(medthick)), ///
        ylabel(, nogrid) ytitle("ln(ENTERT)") xtitle("INCOME")  ///
       title("POE5 Figure 8.8(b)") subtitle("Log-linear model") ///
       legend(off) name(fig8_8b, replace)
reg lentert income college advanced

estat hettest income, iid
estat imtest, white

predict lehat, resid
gen lehat2 = lehat^2
reg lehat2 income
di "0.05 critical value " invchi2tail(e(df_m),.05)
di "BP test " e(N)*e(r2)

reg lehat2 (c.income college advanced)##(c.income college advanced)
di "White test " e(N)*e(r2)
di "0.05 critical value " invchi2tail(e(df_m),.05)

/*---POE5 Example 8.9---*/
* The Marketing Example Revisited
clear all
use stata/coke
summarize

* OLS with inconsistent std errors
quietly regress coke pratio disp_coke disp_pepsi
estimates store ols

* White's test
estat imtest, white
   
* White's test manually
predict ehat, res
gen ehat2 = ehat^2

qui regress ehat2 (c.pratio disp_coke disp_pepsi)#  ///
                  (c.pratio disp_coke disp_pepsi)
di "NR2 = " e(N)*e(r2)
di "df  = " e(df_m)
di "chi-sq 0.95 = " invchi2tail(e(df_m),.05)

* OLS with HCCME std errors
reg coke pratio disp_coke disp_pepsi, vce(robust)
estimates store robust

predict p, xb
gen var = p*(1-p)
summarize p var

* OLS, omitting observations with negative variances
quietly reg coke pratio disp_coke disp_pepsi [aweight=1/var] if  p > 0
estimates store omit

* OLS, where all p<.01 are truncated to be equal .01
gen pcut = cond(p<.01,.01,p)
label var pcut "phat truncated below 0.01"
gen varcut = pcut*(1-pcut)
quietly reg coke pratio disp_coke disp_pepsi [aweight=1/varcut] 
estimates store trunc

esttab ols robust omit trunc, b(%7.4f) se(%7.4f) scalars(N F)   ///
       mtitles("OLS" "Robust" "Omit" "Trunc")

/*---POE5 Example 8.10---*/
* Alternative Robust Standard Errors in the Food Expenditure Model
clear all
use stata/food
reg food_exp income, vce(robust)
estimates store m1

reg food_exp income, vce(hc2)
estimates store m2

reg food_exp income, vce(hc3)
estimates store m3

reg food_exp income
estimates store m4

esttab m1 m2 m3 m4, b(%7.4f) se(%7.4f) scalars(N F)          ///
     nostar mtitles("HC1" "HC2" "HC3" "OLS wrong")
 
esttab m1 m2 m3 m4, b(%7.4g) ci(%7.4g) compress scalars(N F) ///
     nostar mtitles("HC1" "HC2" "HC3" "OLS wrong")

/*---POE5 Appendix 8E---*/	 

clear all
global numsims 1000
global col5 0

* Program for finite-sample properties of OLS
program heterotest, rclass
    version 15.1  
    drop _all
    set obs $numobs
    generate double x2 = runiform(1,5)
    generate double x3 = runiform(1,5)

if $col5==0 {
    generate double hz = exp(1 + $het1*x2 + 0*x3)/$eta
    }

    else {
    generate double hz = runiform(1,11)/$eta
    }

    generate double sig = 3*hz
    generate double y = 5 + 1*x2 + 0*x3 + sig*rnormal() 
 
* ols
    regress y x2 x3
    return scalar b2 = _b[x2]
    return scalar b2se = _se[x2]

* ols robust
    regress y x2 x3, vce(robust)
    return scalar b2robse = _se[x2]

* BP test
    predict eols, resid
    gen eols2 = eols^2
    reg eols2 x2 x3
    return scalar bpnr2 = e(N)*e(r2)

* fgls
    gen leols2 = ln(eols2)
    reg leols2 x2 x3
    gen h0 = exp(_b[x2]*x2 + _b[x3]*x3) 
    regress y x2 x3 [weight=1/h0]
    return scalar b2f = _b[x2]
    return scalar b2fse = _se[x2]

* fgls robust 
    regress y x2 x3 [weight=1/h0], vce(robust)
    return scalar b2frser = _se[x2]
	
* gls
    gen h = x2 
    regress y x2 x3 [weight=1/h]
    return scalar b2g = _b[x2]
    return scalar b2gse = _se[x2]

* gls robust 
    regress y x2 x3 [weight=1/h], vce(robust)
    return scalar b2grser = _se[x2]
    
    end

matrix t8 = J(17,5,.)

    forvalues c=1(1)5 {
        if `c'==1 {

        * table 8e column 1
        global numobs 100
        global het1 0
        global eta 2.7182818 
        global seed 1234567
        }

        else if `c'==2 {
        * table 8e column 2
        global numobs 100
        global het1 .3
        global eta 6.938608 
        global seed 1010101
        }

        else if `c'==3 {
        * table 8e column 3
        global numobs 100
        global het1 .5
        global eta 13.8982 
        global seed 3030303
        }

        else if `c'==4 {
       * table 8e column 4
       global numobs 5000
       global het1 .5
       global eta 14.25737 
       global seed 7654321
       }

       else if `c'==5 {
       * table 8e column 5
       global numobs 5000
       global het1 .5
       global eta 6.025252
       global col5 1
       global seed 5050505
        }

simulate b2r = r(b2) b2fr = r(b2f) b2gr = r(b2g)  /// ols fgls gls estimates
     b2ser = r(b2se) b2robser=r(b2robse)               /// ols & robust ols se
     b2fser = r(b2fse) b2frserr = r(b2frser)           /// fgls & fgls robust se
     b2gser = r(b2gse) b2grserr = r(b2grser)           /// gls & gls robust se
     bpnr2r = r(bpnr2),                                /// BP test
     reps($numsims) saving(hetero, replace)            ///
    seed($seed) nolegend nodots: heterotest
       
matrix t8[1,`c'] = $numobs        // sample size
matrix t8[2,`c'] = $het1          // alpha2 controls hetero

qui sum b2r                       // ols estimates
matrix t8[3,`c']=r(mean)          // mean  of ols estimates
matrix t8[4,`c']=r(sd)            // sd of ols estimates
scalar sdols = r(sd)              // saving sd of ols estimates

qui sum b2ser                     // ols std error
matrix t8[5,`c'] = r(mean)        // mean of ols std error

qui sum b2robser                  // ols rob std err
matrix t8[6,`c'] = r(mean)        // mean ols rob std err

* BP rejection = 1
gen bpnr2rej = chi2tail(2,bpnr2r) < 0.05
qui sum bpnr2rej                  // bp rejection 
matrix t8[7,`c'] = r(mean)        // mean bp rejection

qui sum b2fr                      // fgls estimates
matrix t8[8,`c'] = r(mean)        // mean of fgls estimates
matrix t8[9,`c'] = r(sd)          // sd of fgls estimates
scalar sdfgls = r(sd)             // saving sd fgls
matrix t8[10,`c'] = sdfgls/sdols  // ratio of fgls sd to ols sd

qui sum b2fser                    // fgls std err
matrix t8[11,`c'] = r(mean)       // mean fgls std err

qui sum b2frser                   // fgls rob std err
matrix t8[12,`c'] = r(mean)       // mean fgls rob std err

qui sum b2gr                      // gls estimates
matrix t8[13,`c'] = r(mean)       // mean gls estimates
matrix t8[14,`c'] = r(sd)         // sd of gls estimates
scalar sdgls = r(sd)              // saving sd of gls estimates
matrix t8[15,`c'] = sdgls/sdols   // ratio of gls sd to ols sd

qui sum b2gser                    // gls std err
matrix t8[16,`c'] = r(mean)       // mean gls std err

qui sum b2grser                   // gls rob std err
matrix t8[17,`c'] = r(mean)       // mean gls rob std err

         }

clear
svmat t8
matrix list t8, format(%9.4f)

matrix rowname t8 = N alpha2 b2 sdb2 seb2 robseb                        ///
                    rej b2hat sdb2hat2 sdb2hat2ratio sebhat2 robsebhat2 ///
                    b2hat sdb2hat sdb2hatratio seb2hat robseb2hat
putexcel set table8e1.xlsx, replace
putexcel A1 = matrix(t8), rownames
